"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const EventStore_1 = require("../EventStore/EventStore");
const Persistence_1 = require("../Persistence/Persistence");
var ReadModel;
(function (ReadModel) {
    ReadModel.readModelGenerator = (readModel, reducer, events) => {
        events.forEach((event) => {
            readModel = reducer(readModel, event);
        });
        return readModel;
    };
    ReadModel.maybeSyncReadModel = (es, readModel, reducer) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let solidityEventCount = (yield es.solidityEventCount()).toNumber();
        return Persistence_1.Persistence.LocalStore.getItem(readModel.ReadModelStoreKey)
            .then((_readModel) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!_readModel) {
                _readModel = readModel;
            }
            if (_readModel.LastEvent === solidityEventCount) {
                return false;
            }
            let events = yield EventStore_1.EventStore.readEvents(es, _readModel.LastEvent || 0);
            let updatedReadModel = ReadModel.readModelGenerator(_readModel, reducer, events);
            return Persistence_1.Persistence.LocalStore.setItem(updatedReadModel.ReadModelStoreKey, updatedReadModel);
        }));
    });
    ReadModel.flatten = (data) => {
        var result = {};
        function recurse(cur, prop) {
            if (Object(cur) !== cur) {
                result[prop] = cur;
            }
            else if (Array.isArray(cur)) {
                for (var i = 0, l = cur.length; i < l; i++)
                    recurse(cur[i], prop + "[" + i + "]");
                if (l == 0)
                    result[prop] = [];
            }
            else {
                var isEmpty = true;
                for (var p in cur) {
                    isEmpty = false;
                    recurse(cur[p], prop ? prop + "." + p : p);
                }
                if (isEmpty && prop)
                    result[prop] = {};
            }
        }
        recurse(data, "");
        return result;
    };
    ReadModel.unflatten = (data) => {
        if (Object(data) !== data || Array.isArray(data))
            return data;
        var regex = /\.?([^.\[\]]+)|\[(\d+)\]/g, resultholder = {};
        for (var p in data) {
            var cur = resultholder, prop = "", m;
            while (m = regex.exec(p)) {
                cur = cur[prop] || (cur[prop] = (m[2] ? [] : {}));
                prop = m[2] || m[1];
            }
            cur[prop] = data[p];
        }
        return resultholder[""] || resultholder;
    };
})(ReadModel = exports.ReadModel || (exports.ReadModel = {}));
