"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EventTypes_1 = require("../EventTypes/EventTypes");
const { find, filter, forIn, extend } = require('lodash');
var Transactions;
(function (Transactions) {
    Transactions.getPropFromSchema = (propType, value) => {
        switch (propType) {
            case 'String': return value.toString();
            case 'BigNumber': return value.toNumber();
            default: throw Error(`UNKNWON propType for value '${value}'. Make sure your schema is up to date.`);
        }
    };
    Transactions.eventFromLog = (log) => {
        let schema = EventTypes_1.EventTypes.TruffleEventSchema[log.event];
        let event = {};
        forIn(log.args, (value, key) => {
            let prop = Transactions.getPropFromSchema(schema[key], value);
            extend(event, {
                [key]: prop
            });
        });
        return event;
    };
    Transactions.eventsFromTransaction = (tx) => {
        let allEvents = tx.logs.map((log) => {
            return Transactions.eventFromLog(log);
        });
        return allEvents;
    };
    Transactions.transactionEventsToEventObject = (events) => {
        let eventObjs = filter(events, (evt) => {
            return evt.Id !== undefined;
        });
        eventObjs.forEach((eventObj) => {
            let propIndex = 0;
            while (propIndex < eventObj.PropertyCount) {
                let eventProp = find(events, (evt) => {
                    return evt.EventPropertyIndex === propIndex && evt.EventIndex === eventObj.Id;
                });
                let eventPropObj = EventTypes_1.EventTypes.solidityEventPropertyToObject(eventProp);
                extend(eventObj, eventPropObj);
                propIndex++;
            }
        });
        return eventObjs;
    };
    Transactions.transactionToEventCollection = (tx) => {
        let events = Transactions.eventsFromTransaction(tx);
        let eventCollection = Transactions.transactionEventsToEventObject(events);
        return eventCollection;
    };
})(Transactions = exports.Transactions || (exports.Transactions = {}));
